;****************************************************************************
;RX_BLOQUEADOR_V5
;RECEPTOR P/ BLOQUEADOR DE VECULOS (PRESENA) VERSO 5
;USA PROTOCOLO  DO TIPO ROLLING CODE (MUDA OS VALORES DE CADA TRANSMISSO)
;Autor: Claudio Lrios
;Data: 12/09/2020
;Funciona em conjunto com transmissor "TX_BLOQUEADOR_V2/V3"
;Permite gravar at 18 TX_BLOQUEADOR_V2 (com serial number diferentes)
;ATENO: No funciona com nenhum tipo de TX comerciais (os 'chaveirinhos')
;mesmo os que possuam circuito integrados como o HCS300,301,200,201,etc.
;Necessita montar os 'TX_BLOQUEADOR_V2' e grava-los com firmware fornecido.
;Sobre o funcionamento:
;A cada tansmisso ser enviado um cdigo diferente de 64 bits. Todos os bits
;so encriptados por uma KEY tambm de 64 bits.
;Para gravar um tx:
;1) Aperte o boto 'gravar_ser' do RX (receptor). Neste momento dever acender
;o led de programao por 20 segundos.
;2) No tx (transmissor): Aperte  o boto 'bt_env_ser'(ou coloque jumper) .
;O tx transmitir o serial number para o receptor. O led de programao ir
;apagar indicando que foi gravado o serial number no receptor. Pare de pressi-
;onar o boto 'bt_env_ser'( ou se for jumper, retire ele).
;3) Retire e recoloque a bateria para iniciar uma transmisso. Voc deve
;conferir que neste momento, o receptor acione o rele de sada.
;Importante: No receptor, o boto 'gravar_ser' tem mais uma funo.
;Se ele for apertado continuamente por mais de 10 segundos ir apagar todos os
;controles gravados na EEprom do PIC.
;Nota: O boto 'bt_env_ser' do tx serve somente para enviar o 'serial number'
;para o receptor em modo de gravao.
;Portanto, dever ficar alojado dentro da caixa do tx (podendo ser acessado
;talvez atravs de um pequeno furo, usando a ponta de um clips de papel para
;aperta-lo).
;Opcionalmente, pode ser usado um jumper ao invs de boto nesta funo.
;Altere a Key tanto no receptor como no transmissor e mantenha seu valor si-
;giloso (secreto).
;Importante: A Key deve ser igual nos txs e no rx. O serial number do tx deve
;ser diferente para cada tx (escolha a gosto os valores).
;Use Great Cow Basic Compiler (0.98.06 2019-06-12 (Windows 32 bit))
;IMPORTANTE:
;O PIC  muito sensvel a pulsos de alta tenso podendo resetar ou se perder
;na sequncia de programa. No instale o dispositivo final perto de fios com
;alta tenso, como fios de cabos de velas, ignio eletronica, bobinas ou
;distribuidor. Pegue a alimentao usando 2 fios direto da bateria. Use caixa
;metalica aterrada para acondicionar a montagem.
;Este arquivo  parte integrante do blog Picsource
;Uso para fins didticos
;
;Nota: Para compilar este arquivo usando MPLAB-IDE utilize a opo 
;'Disable Case Sensitivity'.
;Abra o Mplab-Ide e v na barra de ferramenta superior e clique em 
;'Projects'/'Build Options'/'bloq_pres_v5.asm/ clique na caixa 
;'Disable Case Sensitivity'.
;*****************************************************************************'
;Program compiled by Great Cow BASIC (0.98.06 2019-06-12 (Windows 32 bit))
;********************************************************************************

;Set up the assembler options (Chip type, clock source, other bits and pieces)
 LIST p=16F628A, r=DEC
#include <P16F628A.inc>
 __CONFIG _DATA_CP_OFF & _LVP_OFF & _MCLRE_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT
 errorlevel - 302
;********************************************************************************

;Set aside memory locations for variables
ANSEL	EQU	32
AUXA	EQU	33
AUXB	EQU	34
AUXC	EQU	35
AUXD	EQU	36
AUXE	EQU	37
BUFFER_KEY	EQU	327
BUFFER_SER	EQU	318
CNTAP	EQU	38
CNTINC	EQU	39
CNTINC_H	EQU	40
CNTPROG	EQU	41
CNTPROG_E	EQU	44
CNTPROG_H	EQU	42
CNTPROG_U	EQU	43
CONT_ENK0	EQU	45
CONT_ENK1	EQU	46
CONT_ENK2	EQU	47
DELAYTEMP	EQU	112
DELAYTEMP2	EQU	113
END1	EQU	48
HC	EQU	49
LC	EQU	50
PROCURA_GPOS	EQU	51
PROCURA_TX_EEP	EQU	52
QB	EQU	53
SYSARRAYTEMP1	EQU	54
SYSARRAYTEMP2	EQU	55
SYSARRAYTEMP3	EQU	56
SYSBITVAR0	EQU	57
SYSBYTETEMPA	EQU	117
SYSBYTETEMPB	EQU	121
SYSBYTETEMPX	EQU	112
SYSLONGTEMPA	EQU	117
SYSLONGTEMPA_E	EQU	120
SYSLONGTEMPA_H	EQU	118
SYSLONGTEMPA_U	EQU	119
SYSLONGTEMPB	EQU	121
SYSLONGTEMPB_E	EQU	124
SYSLONGTEMPB_H	EQU	122
SYSLONGTEMPB_U	EQU	123
SYSSTRINGA	EQU	119
SYSTEMP1	EQU	58
SYSTEMP2	EQU	59
SYSWAITTEMPMS	EQU	114
SYSWAITTEMPMS_H	EQU	115
SYSWORDTEMPA	EQU	117
SYSWORDTEMPA_H	EQU	118
SYSWORDTEMPB	EQU	121
SYSWORDTEMPB_H	EQU	122
TEMP_DESL	EQU	60
TEMP_DESL_H	EQU	61
TEMP_S	EQU	62
TEMP_SIRENE_ON	EQU	63
VALOR1	EQU	64

;********************************************************************************

;Alias variables
ACCA	EQU	327
ACCA_H	EQU	328
ACCA_U	EQU	329
ACCA_E	EQU	330
ACCB	EQU	331
ACCB_H	EQU	332
ACCB_U	EQU	333
ACCB_E	EQU	334
EEADDRESS	EQU	155
EEDATAVALUE	EQU	154
SYSBUFFER_KEY_0	EQU	327
SYSBUFFER_KEY_1	EQU	328
SYSBUFFER_KEY_2	EQU	329
SYSBUFFER_KEY_3	EQU	330
SYSBUFFER_KEY_4	EQU	331
SYSBUFFER_KEY_5	EQU	332
SYSBUFFER_KEY_6	EQU	333
SYSBUFFER_KEY_7	EQU	334
SYSBUFFER_SER_0	EQU	318
SYSBUFFER_SER_1	EQU	319
SYSBUFFER_SER_2	EQU	320
SYSBUFFER_SER_3	EQU	321
SYSBUFFER_SER_4	EQU	322
SYSBUFFER_SER_5	EQU	323
SYSBUFFER_SER_6	EQU	324
SYSBUFFER_SER_7	EQU	325

;********************************************************************************

;Vectors
	ORG	0
	pagesel	BASPROGRAMSTART
	goto	BASPROGRAMSTART
	ORG	4
	retfie

;********************************************************************************

;Start of program memory page 0
	ORG	5
BASPROGRAMSTART
;Call initialisation routines
	call	INITSYS

;Start of the main program

;BIT
;#define rf_in     PORTA.4 'pino 3 'entrada de sinal do rx
;Dir     rf_in     in
	banksel	TRISA
	bsf	TRISA,4
;#define led       PORTB.1 'pino 7 'led de gravao/apagamento
;Dir     led       out
	bcf	TRISB,1
;#define btn       PORTB.0 'pino 6 'boto gravar/apagar tudo
;Dir     btn       in
	bsf	TRISB,0
;#define rele      PORTA.2 'pino 1 1rele
;Dir     rele      out
	bcf	TRISA,2
;#define led_ad    PORTB.3 'pino 9 'led de advertencia
;Dir     led_ad    out
	bcf	TRISB,3
;#define led_bloq  PORTB.2 'pino 8 'led bloqueado
;Dir     led_bloq  out
	bcf	TRISB,2
;#define in_p15    PORTA.0 'pino 17 sensor de chave contato ligada
;Dir     in_p15    in
	bsf	TRISA,0
;#define sirene    PORTA.1 'pino 18 'led de gravao/apagamento
;Dir     sirene    out
	bcf	TRISA,1
;chave secreta de Encriptao (Secret Encriptacion Key)
;Use os mesmos valores no tx e no rx
;#define KEY0  0X12
;#define KEY1  0X34
;#define KEY2  0X56
;#define KEY3  0X78
;#define KEY4  0X9A
;#define KEY5  0XBC
;#define KEY6  0XDE
;#define KEY7  0XF0
;#define N_ROTATE 64  'rotao do encript
;Dim fled_sir as bit: fled_sir = 0
	banksel	SYSBITVAR0
	bcf	SYSBITVAR0,0
;Dim fled_ad  as bit: fled_ad = 0
	bcf	SYSBITVAR0,1
;Dim flag_c   as bit
;Dim flag0    as bit: flag0 = 0'
	bcf	SYSBITVAR0,3
;Dim flag_ok  as bit: flag_ok = 0'
	bcf	SYSBITVAR0,4
;Dim flag_r   as bit: flag_r = 0'
	bcf	SYSBITVAR0,5
;Dim flag_inc as bit: flag_inc = 0''flag incremento do contador encriptao EEprom
	bcf	SYSBITVAR0,6
;Dim buffer_Ser(8)'buffer de recepo da transmisso
;Dim buffer_Key(8)'buffer para KEY de encriptao/decriptao
;Dim hc as byte ' contador do estado da entrada rf_in em '1'
;Dim lc as byte ' contador do estado da entrada rf_in em '0'
;Dim qb as byte ' contador de bits recebidos
;Dim auxa as byte 'contadores auxiliares
;Dim auxb as byte '
;Dim auxc as byte '
;Dim auxd as byte '
;Dim auxe as byte
;Dim temp_s as byte:  temp_s = 0'auxiliar do tempo da sirene ligada
	clrf	TEMP_S
;Dim temp_sirene_on as byte:  temp_sirene_on = 0'conta minutos da sirene ligada
	clrf	TEMP_SIRENE_ON
;Dim cont_enk0 as byte 'salva contador encriptao lower
;Dim cont_enk1 as byte 'salva contador encriptao higher
;Dim cont_enk2 as byte 'salva contador encriptao upper
;Dim cntap as byte:  cntap = 0'conta tempo para apagamento da eeprom
	clrf	CNTAP
;Dim cntinc as word: cntinc = 0'conta tempo para incrementar contador de encriptao
	clrf	CNTINC
	clrf	CNTINC_H
;Dim cntprog as long : cntprog = 0'conta tempo do led aceso em modo de programao
	clrf	CNTPROG
	clrf	CNTPROG_H
	clrf	CNTPROG_U
	clrf	CNTPROG_E
;Dim accA as long Alias buffer_Key(3),buffer_Key(2),buffer_Key(1),buffer_Key(0)'reuso
;Dim accB as long Alias buffer_Key(7),buffer_Key(6),buffer_Key(5),buffer_Key(4)'reuso
;Dim temp_desl as word: temp_desl = 0' conta tempo para desligamento do rele
	clrf	TEMP_DESL
	clrf	TEMP_DESL_H
;*****************************************************************************
;Tabela com os valores de substituio usados na decriptao de dados
;******************************************************************************'
;*****************************************************************************
;Rotina de escrita na EEprom interna do PIC
;******************************************************************************'
;*****************************************************************************
;Rotina de Delays
;******************************************************************************'
;*****************************************************************************
;Rotina de teste do boto de gravao/sincronizao ou apagamento
;******************************************************************************'
;#define TEMPO_APAGAMENTO 100
;*****************************************************************************
;Rotina de Decriptao dos Dados Recebidos
;*******************************************************************************
;Rotina de decodificao utiliza uma chave secreta (key) de 64 bits.
;Primeiro os dados recebidos sofrem uma substituio de valores a partir de uma
;tabela. Em seguida rotaciona todos os 8 bytes para esquerda em 1 bit. Ento 
;feita a aplicao de XOR com a chave. O processo se repete pelo nmero de rota-
;es estabelecido. No final, temos o valor do serial number (4 bytes) + o
;contador encripta (3 bytes) + situao dos botes (1 byte) . '
;*****************************************************************************
;Rotina de Desligamento do Modo de Programao de Tx's por tempo
;******************************************************************************'
;#define TEMPO_MAX_PROG_MODE 220000 'cerca de uns 20 segundos c/ led ligado
;*****************************************************************************
;Rotina de Incrementos de Registradores da Recepo de Dados
;******************************************************************************'
;*****************************************************************************
;Rotina de Recepo de Dados do Receptor RF
;******************************************************************************'
;#define NUMERO_BITS_A_RECEBER 64
;#define TMAX 20 '20
;*****************************************************************************
;Verifica se tx foi gravado e retorna seu endereo na EEprom do PIC
;******************************************************************************'
;*****************************************************************************
;Rotina de Procura por Posies Vazias na EEprom (para gravar novo tx)
;******************************************************************************'
;*****************************************************************************
;Rotina de Gravao/ressincronizao de Transmissores
;******************************************************************************'
;Quando em 'modo gravao' (led aceso) ir aguardar a recepo do serial
;number do tx a gravar. O usurio deve apertar somente o boto 'bt_env_ser'
;do tx para que este receptor faa o aprendizado e grave na eeprom.         '
;*****************************************************************************
;Rotina de Acionamento do rele
;******************************************************************************'
;#define MAX_DESSINCRONIZACAO 1'ajusta taxa de dessincronizao mxima
;*****************************************************************************
;Rotina de Deteco do Fim das Transmisses p/ Salvar Valor do Contador Encript
;******************************************************************************'
;Somente ao termino de uma transmisso (curta ou longa dependendo do tempo que
;o boto do tx fique apertado pelo usurio)  que ser atualizado o contador
;encript. Cada vez que recebe uma transmisso correta o contador cntinc  zerado.
;Quando para a transmisso ele consegue incrementar a ponto de permitir que seja
;feita a atualizao do novo valor do contador encript.  '
;#define VALOR_DELAY_INCREMENTO 2000
;*****************************************************************************
;Rotina de Desligamento do rele com indicao por leds
;******************************************************************************'
;#define VALOR_ADVERTENCIA  171  '90 segundos
;#define VALOR_BLOQUEIO     342 'VALOR_ADVERTENCIA * 2'o dobro do tempo do anterior
;******************************************************************************
;Rotina de Desligamento da sirene aps disparo por tempo
;*******************************************************************************/
;#define VALOR_DESLIGA_SIRENE  10 '10 minutos
;#define VALOR_UM_MINUTO  114
;******************************************************************************
;Rotina de Teste para o Disparo da Sirene
;
;Quando o sensor de presena estiver longe do veculo e tiver passado
;o tempo de bloqueio, ento, ao ligar a chave de contato do veculo
;ir disparar a sirene por um tempo determinado e ir parar.
;Se a chave de contato for ligada novamente ento ocorrer novo
;disparo da sirene.
;*******************************************************************************/
;*****************************************************************************
;Rotina Main
;******************************************************************************'
;OPTION_REG = 0'timer 0 sem prescaller
	banksel	OPTION_REG
	clrf	OPTION_REG
;T1CON = 0b00110001''liga TIMER 1 prescaller/8
	movlw	49
	banksel	T1CON
	movwf	T1CON
;TMR1IF = 0'reseta flag de overflow do timer 1
	bcf	PIR1,TMR1IF
;CMCON = 7''desliga comparadores
	movlw	7
	movwf	CMCON
;PORTA = 0
	clrf	PORTA
;TRISA = 0b11111001
	movlw	249
	banksel	TRISA
	movwf	TRISA
;PORTB = 0
	banksel	PORTB
	clrf	PORTB
;TRISB = 0b11110001
	movlw	241
	banksel	TRISB
	movwf	TRISB
;ANSEL = 0'desliga conversor AD
	banksel	ANSEL
	clrf	ANSEL
;lc = 0'contador da situao da entrada in_rf em '0'
	clrf	LC
;hc = 0'contador da situao da entrada in_rf em '1'
	clrf	HC
;flag0 = 0' flag indicador da situao da entrada in_rf
	bcf	SYSBITVAR0,3
;qb = 0'contador de bits recebidos
	clrf	QB
;flag_ok = 0'reseta flag de recepo ok de tx
	bcf	SYSBITVAR0,4
;flag_r = 0'reseta flag de segunda recepo de tx
	bcf	SYSBITVAR0,5
;EPRead 127,auxc
	movlw	127
	banksel	EEADDRESS
	movwf	EEADDRESS
	banksel	STATUS
	call	SYSEPREAD
	banksel	EEDATAVALUE
	movf	EEDATAVALUE,W
	banksel	AUXC
	movwf	AUXC
;if auxc = 1 then
	decf	AUXC,W
	btfss	STATUS, Z
	goto	ENDIF1
;sirene = 1
	bsf	PORTA,1
;rele = 0 'desliga rele
	bcf	PORTA,2
;temp_s = 0
	clrf	TEMP_S
;temp_sirene_on = 0 'zera contador de tempo de desligamento
	clrf	TEMP_SIRENE_ON
;fled_sir = 1 'seta flag de tempo de sirene disparada
	bsf	SYSBITVAR0,0
;led_ad = 0 'desliga led_gravar de advertncia
	bcf	PORTB,3
;fled_ad = 0 'reseta flag do led de advertncia
	bcf	SYSBITVAR0,1
;led_bloq = 1 'liga led indicador de bloqueio
	bsf	PORTB,2
;temp_desl = 0 'zera contador de tempo de desligamento
	clrf	TEMP_DESL
	clrf	TEMP_DESL_H
;end if
ENDIF1
;*****************************************************************************
;Lao de Repetio
;******************************************************************************'
;do
SysDoLoop_S1
;testa_botao''testa boto gravao/apagamento/ressincronizao
	call	TESTA_BOTAO
;temporiza_prog_mode''sai do modo de programao por tempo
	call	TEMPORIZA_PROG_MODE
;receber_dados''realiza a recepo dos dados vindos do tx
	call	RECEBER_DADOS
;if led = 1 then 'testa se modo de gravao de tx est em andamento
	btfss	PORTB,1
	goto	ELSE2_1
;gravar_syn_tx 'rotina de gravao/ressincronizao de tx
	call	GRAVAR_SYN_TX
;else'  modo normal
	goto	ENDIF2
ELSE2_1
;decriptar''decodifica a transmisso recebida
	call	DECRIPTAR
;acionar_rele''aciona rele se encontrou serial number do tx j gravado
	call	ACIONAR_RELE
;end if
ENDIF2
;manter a sub 'inc_cont_eep' logo aps sub 'acionar_rele' p/ aproveitar valor de 'auxd'
;inc_cont_eep   'incrementa contador de encriptao na EEprom
	call	INC_CONT_EEP
;temporiza_rele 'temporiza rele para desligamento caso no receba tx
	call	TEMPORIZA_RELE
;disparar_sirene 'testa se deve disparar sirene
	call	DISPARAR_SIRENE
;temporiza_sirene 'temporiza sirene
	call	TEMPORIZA_SIRENE
;loop
	goto	SysDoLoop_S1
SysDoLoop_E1
;end
	goto	BASPROGRAMEND
;------------------------------------------------------------------------------
;fim do programa
;------------------------------------------------------------------------------
BASPROGRAMEND
	sleep
	goto	BASPROGRAMEND

;********************************************************************************

ACIONAR_RELE
;Se o controle remoto transmissor tiver os seus botes apertados por mais de
;5 vezes e estando longe do receptor (portanto, sem acionar o receptor) ento
;ficar dessincronizado. Na primeira vez que acionar o boto perto do receptor
;no  ligar o rele, mas ir ressincronizar ao trmino da recepo. A proxima
;transmisso, se correta, ir acionar o rele. Isto previne ataque a 'fora
;bruta', pois necessitar 2 transmisses corretas sucessivas para ativar o
;rele. O objetivo do contador de encriptao tambm ser salvo no receptor 
;poder bloquear as transmisses corretas anteriores. Isto evita que algum
;escaneie a transmisso e tente usa-la para acionar o receptor.
;Contador encript do tx recebido no buffer_Ser(4),(5),(6).          '
;if flag_ok = 1 then
	btfss	SYSBITVAR0,4
	goto	ENDIF33
;auxd = procura_tx_eep'auxd dever ser usada p/ conter posio do tx nesta rotina
	call	FN_PROCURA_TX_EEP
	movf	PROCURA_TX_EEP,W
	movwf	AUXD
;e na prxima: sub inc_cont_eep (cuidado com posio da sub)
;if auxd < 128 then'foi encontrado um tx cadastrado (auxd tem o endereo dele)
	movlw	128
	subwf	AUXD,W
	btfsc	STATUS, C
	goto	ENDIF34
;Salva valor do contador encript recebido do tx p/ depois salvar na EEprom
;cont_enk0 = buffer_Ser(4)''contador encript lower
	banksel	SYSBUFFER_SER_4
	movf	SYSBUFFER_SER_4,W
	banksel	CONT_ENK0
	movwf	CONT_ENK0
;cont_enk1 = buffer_Ser(5)''contador encript higher
	banksel	SYSBUFFER_SER_5
	movf	SYSBUFFER_SER_5,W
	banksel	CONT_ENK1
	movwf	CONT_ENK1
;cont_enk2 = buffer_Ser(6)''contador encript upper
	banksel	SYSBUFFER_SER_6
	movf	SYSBUFFER_SER_6,W
	banksel	CONT_ENK2
	movwf	CONT_ENK2
;flag_inc=1' 'sinaliza p/ processo de incremento do contadorEncript
;cntinc = 0''reinicia tempo de espera ps termino de recepo
	clrf	CNTINC
	clrf	CNTINC_H
;accA= Varivel de 32 bits p/ receber o valor do contador encript deste rx
;EPRead auxd + 4, accA 'obtem cnt lower do receptor
	movlw	4
	addwf	AUXD,W
	banksel	EEADDRESS
	movwf	EEADDRESS
	banksel	STATUS
	call	SYSEPREAD
	banksel	EEDATAVALUE
	movf	EEDATAVALUE,W
	banksel	ACCA
	movwf	ACCA
	clrf	ACCA_H
	clrf	ACCA_U
	clrf	ACCA_E
;EPRead auxd + 5, accA_H 'obtem cnt higher do receptor
	movlw	5
	banksel	AUXD
	addwf	AUXD,W
	banksel	EEADDRESS
	movwf	EEADDRESS
	banksel	STATUS
	call	SYSEPREAD
	banksel	EEDATAVALUE
	movf	EEDATAVALUE,W
	banksel	ACCA_H
	movwf	ACCA_H
;EPRead auxd + 6, accA_U 'obtem cnt upper
	movlw	6
	banksel	AUXD
	addwf	AUXD,W
	banksel	EEADDRESS
	movwf	EEADDRESS
	banksel	STATUS
	call	SYSEPREAD
	banksel	EEDATAVALUE
	movf	EEDATAVALUE,W
	banksel	ACCA_U
	movwf	ACCA_U
;accA_E = 0
	clrf	ACCA_E
;accB= Varivel de 32 bits p/ receber o valor do contador encript do tx
;accB   = cont_enk0'
	banksel	CONT_ENK0
	movf	CONT_ENK0,W
	banksel	ACCB
	movwf	ACCB
	clrf	ACCB_H
	clrf	ACCB_U
	clrf	ACCB_E
;accB_H = cont_enk1'
	banksel	CONT_ENK1
	movf	CONT_ENK1,W
	banksel	ACCB_H
	movwf	ACCB_H
;accB_U = cont_enk2'
	banksel	CONT_ENK2
	movf	CONT_ENK2,W
	banksel	ACCB_U
	movwf	ACCB_U
;accB_E = 0
	clrf	ACCB_E
;Comparao p/ impedir que transmisses corretas anteriores acionem o rele
;Evita que uma transmisso anterior escaneada consiga acionar o rele.
;valor do contador recebido tem que ser maior que o salvo no receptor.
;if accB > accA then
	movf	ACCB,W
	movwf	SysLONGTempB
	movf	ACCB_H,W
	movwf	SysLONGTempB_H
	movf	ACCB_U,W
	movwf	SysLONGTempB_U
	movf	ACCB_E,W
	movwf	SysLONGTempB_E
	movf	ACCA,W
	movwf	SysLONGTempA
	movf	ACCA_H,W
	movwf	SysLONGTempA_H
	movf	ACCA_U,W
	movwf	SysLONGTempA_U
	movf	ACCA_E,W
	movwf	SysLONGTempA_E
	banksel	STATUS
	call	SysCompLessThan32
	btfss	SysByteTempX,0
	goto	ENDIF35
;accA = accB - accA''verifica taxa de dessincronizao
	banksel	ACCA
	movf	ACCA,W
	subwf	ACCB,W
	movwf	ACCA
	movf	ACCB_H,W
	movwf	ACCA_H
	btfss	STATUS,C
	incfsz	ACCA_H,W
	subwf	ACCA_H,F
	movf	ACCB_U,W
	movwf	ACCA_U
	btfss	STATUS,C
	incfsz	ACCA_U,W
	subwf	ACCA_U,F
	movf	ACCB_E,W
	movwf	ACCA_E
	btfss	STATUS,C
	incfsz	ACCA_E,W
	subwf	ACCA_E,F
;if accA <= MAX_DESSINCRONIZACAO then 'testa grau de dessincronizao
	movf	ACCA,W
	movwf	SysLONGTempB
	movf	ACCA_H,W
	movwf	SysLONGTempB_H
	movf	ACCA_U,W
	movwf	SysLONGTempB_U
	movf	ACCA_E,W
	movwf	SysLONGTempB_E
	movlw	1
	movwf	SysLONGTempA
	clrf	SysLONGTempA_H
	clrf	SysLONGTempA_U
	clrf	SysLONGTempA_E
	banksel	STATUS
	call	SysCompLessThan32
	comf	SysByteTempX,F
	btfss	SysByteTempX,0
	goto	ENDIF36
;sirene = 0' desliga sirene se estiver ligada
	bcf	PORTA,1
;temp_sirene_on = 0;//zera contador
	clrf	TEMP_SIRENE_ON
;temp_s = 0;//zera contador auxiliar
	clrf	TEMP_S
;escreve_eep(127,0)'grava zero na posio 127 da eeprom
	movlw	127
	movwf	END1
	clrf	VALOR1
	call	ESCREVE_EEP
;rele = 1''
	bsf	PORTA,2
;fled_ad = 0;//reseta flag do led de advertncia
	bcf	SYSBITVAR0,1
;led_ad = 0''desliga led de advertncia
	bcf	PORTB,3
;led_bloq = 0''desliga led de bloqueio
	bcf	PORTB,2
;temp_desl = 0''zera contador de tempo
	clrf	TEMP_DESL
	clrf	TEMP_DESL_H
;end if
ENDIF36
;flag_inc = 1''sinaliza p/ processo de incremento do contadorEncript
	bsf	SYSBITVAR0,6
;end if'if(accB > accA){
ENDIF35
;end if
ENDIF34
;flag_ok = 0''libera nova recepo
	bcf	SYSBITVAR0,4
;end if
ENDIF33
	return

;********************************************************************************

DECRIPTAR
;if flag_ok = 0 then Exit Sub
	btfss	SYSBITVAR0,4
	return
;
;Carrega valores da Key de Encriptao
;buffer_Key(0) = KEY0'
	movlw	18
	banksel	SYSBUFFER_KEY_0
	movwf	SYSBUFFER_KEY_0
;buffer_Key(1) = KEY1'
	movlw	52
	movwf	SYSBUFFER_KEY_1
;buffer_Key(2) = KEY2'
	movlw	86
	movwf	SYSBUFFER_KEY_2
;buffer_Key(3) = KEY3'
	movlw	120
	movwf	SYSBUFFER_KEY_3
;buffer_Key(4) = KEY4'
	movlw	154
	movwf	SYSBUFFER_KEY_4
;buffer_Key(5) = KEY5'
	movlw	188
	movwf	SYSBUFFER_KEY_5
;buffer_Key(6) = KEY6'
	movlw	222
	movwf	SYSBUFFER_KEY_6
;buffer_Key(7) = KEY7'
	movlw	240
	movwf	SYSBUFFER_KEY_7
;for auxa = 1 to N_ROTATE
	banksel	AUXA
	clrf	AUXA
SysForLoop3
	incf	AUXA,F
;rotina de substituio de valores
;for auxb = 0 to 7
	movlw	255
	movwf	AUXB
SysForLoop4
	incf	AUXB,F
;auxc = buffer_Ser(auxb) and 0x0f'
	movlw	low(BUFFER_SER)
	addwf	AUXB,W
	movwf	FSR
	bankisel	BUFFER_SER
	movlw	15
	andwf	INDF,W
	movwf	AUXC
;auxc = auxc + 1' ajusta a posio correta na tabela
	incf	AUXC,F
;ReadTable TABELA_Z, auxc, auxe
	movf	AUXC,W
	movwf	SYSSTRINGA
	call	TABELA_Z
	movwf	AUXE
;buffer_Ser(auxb) = buffer_Ser(auxb) and 0xf0''zera 4 bits low
	movlw	low(BUFFER_SER)
	addwf	AUXB,W
	movwf	FSR
	bankisel	BUFFER_SER
	movf	INDF,W
	movwf	SysArrayTemp2
	movlw	240
	andwf	SysArrayTemp2,W
	movwf	SysArrayTemp1
	movlw	low(BUFFER_SER)
	addwf	AUXB,W
	movwf	FSR
	bankisel	BUFFER_SER
	movf	SysArrayTemp1,W
	movwf	INDF
;buffer_Ser(auxb) = buffer_Ser(auxb) or auxe'
	movlw	low(BUFFER_SER)
	addwf	AUXB,W
	movwf	FSR
	bankisel	BUFFER_SER
	movf	INDF,W
	movwf	SysArrayTemp2
	iorwf	AUXE,W
	movwf	SysArrayTemp1
	movlw	low(BUFFER_SER)
	addwf	AUXB,W
	movwf	FSR
	bankisel	BUFFER_SER
	movf	SysArrayTemp1,W
	movwf	INDF
;next
	movlw	7
	subwf	AUXB,W
	btfss	STATUS, C
	goto	SysForLoop4
SysForLoopEnd4
;ReadTable nome, pos, destino
;rotate_left(buffer_Ser,8)''rotaciona 1 bits a esquerda
;if (buffer_Ser(7) and 0x80) = 0 then
	movlw	128
	banksel	SYSBUFFER_SER_7
	andwf	SYSBUFFER_SER_7,W
	banksel	SYSTEMP1
	movwf	SysTemp1
	movwf	SysBYTETempA
	clrf	SysBYTETempB
	call	SysCompEqual
	btfss	SysByteTempX,0
	goto	ELSE9_1
;C = 0
	bcf	STATUS,C
;else
	goto	ENDIF9
ELSE9_1
;C = 1
	bsf	STATUS,C
;end if
ENDIF9
;auxb = buffer_Ser(7)
;C = FnLSR(auxb, 7)
;rotate  buffer_Ser(0) left
	banksel	SYSBUFFER_SER_0
	rlf	SYSBUFFER_SER_0,F
;rotate  buffer_Ser(1) left
	rlf	SYSBUFFER_SER_1,F
;rotate  buffer_Ser(2) left
	rlf	SYSBUFFER_SER_2,F
;rotate  buffer_Ser(3) left
	rlf	SYSBUFFER_SER_3,F
;rotate  buffer_Ser(4) left
	rlf	SYSBUFFER_SER_4,F
;rotate  buffer_Ser(5) left
	rlf	SYSBUFFER_SER_5,F
;rotate  buffer_Ser(6) left
	rlf	SYSBUFFER_SER_6,F
;rotate  buffer_Ser(7) left
	rlf	SYSBUFFER_SER_7,F
;loop de encriptao
;for auxb = 0 to 7
	movlw	255
	banksel	AUXB
	movwf	AUXB
SysForLoop5
	incf	AUXB,F
;buffer_Ser(auxb) = buffer_Ser(auxb) xor buffer_Key(auxb)'
	movlw	low(BUFFER_KEY)
	addwf	AUXB,W
	movwf	FSR
	bankisel	BUFFER_KEY
	movf	INDF,W
	movwf	SysArrayTemp1
	movlw	low(BUFFER_SER)
	addwf	AUXB,W
	movwf	FSR
	bankisel	BUFFER_SER
	movf	INDF,W
	movwf	SysArrayTemp3
	xorwf	SysArrayTemp1,W
	movwf	SysArrayTemp2
	movlw	low(BUFFER_SER)
	addwf	AUXB,W
	movwf	FSR
	bankisel	BUFFER_SER
	movf	SysArrayTemp2,W
	movwf	INDF
;next
	movlw	7
	subwf	AUXB,W
	btfss	STATUS, C
	goto	SysForLoop5
SysForLoopEnd5
;next
	movlw	64
	subwf	AUXA,W
	btfss	STATUS, C
	goto	SysForLoop3
SysForLoopEnd3
	return

;********************************************************************************

DELAY50MS
;wait 50 ms'
	movlw	50
	movwf	SysWaitTempMS
	clrf	SysWaitTempMS_H
	goto	Delay_MS

;********************************************************************************

DELAYP
;wait 300 ms'
	movlw	44
	movwf	SysWaitTempMS
	movlw	1
	movwf	SysWaitTempMS_H
	goto	Delay_MS

;********************************************************************************

DISPARAR_SIRENE
;if fled_sir = 1 then
	btfss	SYSBITVAR0,0
	goto	ENDIF57
;if in_p15 = 0 then 'sem sensor de presena e chave contato ligada
	btfsc	PORTA,0
	goto	ENDIF59
;EPRead 127,auxc
	movlw	127
	banksel	EEADDRESS
	movwf	EEADDRESS
	banksel	STATUS
	call	SYSEPREAD
	banksel	EEDATAVALUE
	movf	EEDATAVALUE,W
	banksel	AUXC
	movwf	AUXC
;if auxc = 0 then escreve_eep(127,1)'grava '1'
	movf	AUXC,F
	btfss	STATUS, Z
	goto	ENDIF60
	movlw	127
	movwf	END1
	movlw	1
	movwf	VALOR1
	call	ESCREVE_EEP
ENDIF60
;end if
ENDIF59
;end if
ENDIF57
;EPRead 127,auxc
	movlw	127
	banksel	EEADDRESS
	movwf	EEADDRESS
	banksel	STATUS
	call	SYSEPREAD
	banksel	EEDATAVALUE
	movf	EEDATAVALUE,W
	banksel	AUXC
	movwf	AUXC
;if auxc = 1 then sirene = 1 'dispara sirene
	decf	AUXC,W
	btfsc	STATUS, Z
	bsf	PORTA,1
	return

;********************************************************************************

Delay_MS
	incf	SysWaitTempMS_H, F
DMS_START
	movlw	142
	movwf	DELAYTEMP2
DMS_OUTER
	movlw	1
	movwf	DELAYTEMP
DMS_INNER
	decfsz	DELAYTEMP, F
	goto	DMS_INNER
	decfsz	DELAYTEMP2, F
	goto	DMS_OUTER
	decfsz	SysWaitTempMS, F
	goto	DMS_START
	decfsz	SysWaitTempMS_H, F
	goto	DMS_START
	return

;********************************************************************************

EPWRITE
;Variable alias
;Dim EEAddress Alias EEADR  'erv
;Dim EEDataValue Alias EEDATL_REF
;Disable interrupt
;IntOff
;Select data memory
;Start write
;SET WREN ON
	banksel	EECON1
	bsf	EECON1,WREN
;EECON2 = 0x55
	movlw	85
	movwf	EECON2
;EECON2 = 0xAA
	movlw	170
	movwf	EECON2
;SET WR ON
	bsf	EECON1,WR
;SET WREN OFF
	bcf	EECON1,WREN
;Wait for write to complete
;WAIT WHILE WR ON
SysWaitLoop1
	btfsc	EECON1,WR
	goto	SysWaitLoop1
;SET WREN OFF
	bcf	EECON1,WREN
;Restore interrupt
;IntOn
	banksel	STATUS
	return

;********************************************************************************

ESCREVE_EEP
;EPWrite end1, valor1 '
	movf	END1,W
	banksel	EEADDRESS
	movwf	EEADDRESS
	banksel	VALOR1
	movf	VALOR1,W
	banksel	EEDATAVALUE
	movwf	EEDATAVALUE
	banksel	STATUS
	goto	EPWRITE

;********************************************************************************

GRAVAR_SYN_TX
;if flag_ok = 1 then
	btfss	SYSBITVAR0,4
	goto	ENDIF25
;auxa = 0'
	clrf	AUXA
;for auxb = 0 to 6
	movlw	255
	movwf	AUXB
SysForLoop8
	incf	AUXB,F
;auxa = auxa + buffer_Ser(auxb)''soma  7 bytes recebidos
	movlw	low(BUFFER_SER)
	addwf	AUXB,W
	movwf	FSR
	bankisel	BUFFER_SER
	movf	INDF,W
	addwf	AUXA,F
;next
	movlw	6
	subwf	AUXB,W
	btfss	STATUS, C
	goto	SysForLoop8
SysForLoopEnd8
;if auxa = buffer_Ser(7) then'verifica o 'test sum do oitavo byte  igual a soma'
	banksel	SYSBUFFER_SER_7
	movf	SYSBUFFER_SER_7,W
	banksel	AUXA
	subwf	AUXA,W
	btfss	STATUS, Z
	goto	ENDIF27
;auxd = procura_tx_eep''procura tx na eeprom, se j esta gravado
	call	FN_PROCURA_TX_EEP
	movf	PROCURA_TX_EEP,W
	movwf	AUXD
;if auxd < 128 then'j est gravado,ir somente ressincronizar o contador
	movlw	128
	subwf	AUXD,W
	btfsc	STATUS, C
	goto	ELSE28_1
;for auxa = 4 to 6 'grava 3 bytes do contador na eeprom
	movlw	3
	movwf	AUXA
SysForLoop9
	incf	AUXA,F
;auxb = auxd + auxa
	movf	AUXA,W
	addwf	AUXD,W
	movwf	AUXB
;escreve_eep(auxb,buffer_Ser(auxa))'
	movwf	END1
	movlw	low(BUFFER_SER)
	addwf	AUXA,W
	movwf	FSR
	bankisel	BUFFER_SER
	movf	INDF,W
	movwf	VALOR1
	call	ESCREVE_EEP
	movf	END1,W
	movwf	AUXB
	movlw	low(BUFFER_SER)
	addwf	AUXA,W
	movwf	FSR
	bankisel	BUFFER_SER
	movf	VALOR1,W
	movwf	INDF
;next
	movlw	6
	subwf	AUXA,W
	btfss	STATUS, C
	goto	SysForLoop9
SysForLoopEnd9
;led = 0: delayp: led = 1 : delayp''piscada adicional para ressincronizao
	bcf	PORTB,1
	call	DELAYP
	bsf	PORTB,1
	call	DELAYP
;else
	goto	ENDIF28
ELSE28_1
;auxe = procura_gpos''procura posio livre para gravar
	call	FN_PROCURA_GPOS
	movf	PROCURA_GPOS,W
	movwf	AUXE
;if auxe = 0xff then'no tem mais espao, sinaliza erro com 5 piscadas
	incf	AUXE,W
	btfss	STATUS, Z
	goto	ELSE30_1
;for auxb = 0 to 4
	movlw	255
	movwf	AUXB
SysForLoop10
	incf	AUXB,F
;led = 0 : delayp : led=1: delayp'
	bcf	PORTB,1
	call	DELAYP
	bsf	PORTB,1
	call	DELAYP
;next
	movlw	4
	subwf	AUXB,W
	btfss	STATUS, C
	goto	SysForLoop10
SysForLoopEnd10
;else
	goto	ENDIF30
ELSE30_1
;for auxa = 0 to 6'grava 7 bytes na eeprom
	movlw	255
	movwf	AUXA
SysForLoop11
	incf	AUXA,F
;auxb = auxe + auxa
	movf	AUXA,W
	addwf	AUXE,W
	movwf	AUXB
;escreve_eep(auxb,buffer_Ser(auxa))'
	movwf	END1
	movlw	low(BUFFER_SER)
	addwf	AUXA,W
	movwf	FSR
	bankisel	BUFFER_SER
	movf	INDF,W
	movwf	VALOR1
	call	ESCREVE_EEP
	movf	END1,W
	movwf	AUXB
	movlw	low(BUFFER_SER)
	addwf	AUXA,W
	movwf	FSR
	bankisel	BUFFER_SER
	movf	VALOR1,W
	movwf	INDF
;next
	movlw	6
	subwf	AUXA,W
	btfss	STATUS, C
	goto	SysForLoop11
SysForLoopEnd11
;end if
ENDIF30
;end if
ENDIF28
;led = 0: delayp: led = 1: delayp: led = 0'' 1 piscada para gravao correta
	bcf	PORTB,1
	call	DELAYP
	bsf	PORTB,1
	call	DELAYP
	bcf	PORTB,1
;end if
ENDIF27
;end if
ENDIF25
;flag_ok = 0'libera nova recepo
	bcf	SYSBITVAR0,4
	return

;********************************************************************************

INCREMENTA_HC_LC
;if rf_in = 1 then
	btfss	PORTA,4
	goto	ELSE61_1
;flag0 = 1''flag de estado da entrada rf_in
	bsf	SYSBITVAR0,3
;if hc < 255 then hc = hc + 1 'no deixa passar de 255
	movlw	255
	subwf	HC,W
	btfss	STATUS, C
	incf	HC,F
;else
	goto	ENDIF61
ELSE61_1
;if lc < 255 then lc = lc + 1 'no deixa passar de 255
	movlw	255
	subwf	LC,W
	btfss	STATUS, C
	incf	LC,F
;end if
ENDIF61
;wait 50 ms'delay 50
	return

;********************************************************************************

INC_CONT_EEP
;if flag_inc = 1 then
	btfss	SYSBITVAR0,6
	goto	ENDIF40
;cntinc = cntinc + 1
	incf	CNTINC,F
	btfsc	STATUS,Z
	incf	CNTINC_H,F
;if cntinc > VALOR_DELAY_INCREMENTO then
	movf	CNTINC,W
	movwf	SysWORDTempB
	movf	CNTINC_H,W
	movwf	SysWORDTempB_H
	movlw	208
	movwf	SysWORDTempA
	movlw	7
	movwf	SysWORDTempA_H
	call	SysCompLessThan16
	btfss	SysByteTempX,0
	goto	ENDIF41
;ressincroniza automticamente
;EPRead auxd + 4,auxc
	movlw	4
	addwf	AUXD,W
	banksel	EEADDRESS
	movwf	EEADDRESS
	banksel	STATUS
	call	SYSEPREAD
	banksel	EEDATAVALUE
	movf	EEDATAVALUE,W
	banksel	AUXC
	movwf	AUXC
;if auxc <> cont_enk0 then'salva apenas se valor for diferente
	movf	CONT_ENK0,W
	subwf	AUXC,W
	btfsc	STATUS, Z
	goto	ENDIF42
;escreve_eep(auxd + 4, cont_enk0)''salva cnt lower
	movlw	4
	addwf	AUXD,W
	movwf	END1
	movf	CONT_ENK0,W
	movwf	VALOR1
	call	ESCREVE_EEP
	movf	VALOR1,W
	movwf	CONT_ENK0
;end if
ENDIF42
;EPRead auxd + 5,auxc
	movlw	5
	addwf	AUXD,W
	banksel	EEADDRESS
	movwf	EEADDRESS
	banksel	STATUS
	call	SYSEPREAD
	banksel	EEDATAVALUE
	movf	EEDATAVALUE,W
	banksel	AUXC
	movwf	AUXC
;if auxc <> cont_enk1 then 'salva apenas se valor for diferente
	movf	CONT_ENK1,W
	subwf	AUXC,W
	btfsc	STATUS, Z
	goto	ENDIF43
;escreve_eep(auxd + 5, cont_enk1)''salva cnt higher
	movlw	5
	addwf	AUXD,W
	movwf	END1
	movf	CONT_ENK1,W
	movwf	VALOR1
	call	ESCREVE_EEP
	movf	VALOR1,W
	movwf	CONT_ENK1
;end if
ENDIF43
;EPRead auxd + 6,auxc
	movlw	6
	addwf	AUXD,W
	banksel	EEADDRESS
	movwf	EEADDRESS
	banksel	STATUS
	call	SYSEPREAD
	banksel	EEDATAVALUE
	movf	EEDATAVALUE,W
	banksel	AUXC
	movwf	AUXC
;if auxc <> cont_enk2 then'salva apenas se valor for diferente
	movf	CONT_ENK2,W
	subwf	AUXC,W
	btfsc	STATUS, Z
	goto	ENDIF44
;escreve_eep(auxd + 6, cont_enk2)''salva cnt uppe
	movlw	6
	addwf	AUXD,W
	movwf	END1
	movf	CONT_ENK2,W
	movwf	VALOR1
	call	ESCREVE_EEP
	movf	VALOR1,W
	movwf	CONT_ENK2
;end if
ENDIF44
;flag_inc = 0''impede entrada nesta rotina at uma nova recepo ok
	bcf	SYSBITVAR0,6
;end if
ENDIF41
;end if
ENDIF40
	return

;********************************************************************************

INITSYS
;Set up internal oscillator
;Handle OSCCON1 register for parts that have this register
;Turn off comparator - this is the default setting
;CMCON = 7
	movlw	7
	movwf	CMCON
;Turn off all ports
;PORTA = 0
	clrf	PORTA
;PORTB = 0
	clrf	PORTB
	return

;********************************************************************************

FN_PROCURA_GPOS
;for auxa = 0 to 119
	movlw	255
	movwf	AUXA
SysForLoop14
	incf	AUXA,F
;for auxb = 0 to 7
	movlw	255
	movwf	AUXB
SysForLoop15
	incf	AUXB,F
;auxe = auxa + auxb 'auxe tem o endereo da eeprom
	movf	AUXB,W
	addwf	AUXA,W
	movwf	AUXE
;EPRead auxe, auxc  'auxc tem o conteudo da posio da eeprom
	banksel	EEADDRESS
	movwf	EEADDRESS
	banksel	STATUS
	call	SYSEPREAD
	banksel	EEDATAVALUE
	movf	EEDATAVALUE,W
	banksel	AUXC
	movwf	AUXC
;if auxc <> 0xff  then exit for'sai do lao
	incf	AUXC,W
	btfss	STATUS, Z
	goto	SysForLoopEnd15
;next
	movlw	7
	subwf	AUXB,W
	btfss	STATUS, C
	goto	SysForLoop15
SysForLoopEnd15
;if auxb  = 7 then 'estava 8
	movlw	7
	subwf	AUXB,W
	btfss	STATUS, Z
	goto	ENDIF70
;procura_gpos = auxa'retorna a posio que foi encontrado
	movf	AUXA,W
	movwf	PROCURA_GPOS
;exit sub
	return
;end if
ENDIF70
;auxa = auxa + 6'
	movlw	6
	addwf	AUXA,F
;next
	movlw	119
	subwf	AUXA,W
	btfss	STATUS, C
	goto	SysForLoop14
SysForLoopEnd14
;procura_gpos = 0xff' ' aivsa que no h mais posio para gravar
	movlw	255
	movwf	PROCURA_GPOS
	return

;********************************************************************************

FN_PROCURA_TX_EEP
;procura de 7 em 7 bytes mas compara apenas os 4 primeiros bytes de 7
;for  auxa = 0 to 119
	movlw	255
	movwf	AUXA
SysForLoop12
	incf	AUXA,F
;for  auxb = 0 to 3
	movlw	255
	movwf	AUXB
SysForLoop13
	incf	AUXB,F
;auxe = auxa + auxb 'auxe tem o endereo da eeprom
	movf	AUXB,W
	addwf	AUXA,W
	movwf	AUXE
;EPRead auxe, auxc 'auxc tem o valor lido na eeprom
	banksel	EEADDRESS
	movwf	EEADDRESS
	banksel	STATUS
	call	SYSEPREAD
	banksel	EEDATAVALUE
	movf	EEDATAVALUE,W
	banksel	AUXC
	movwf	AUXC
;if buffer_Ser(auxb) <> auxc then exit for'sai do lao
	movlw	low(BUFFER_SER)
	addwf	AUXB,W
	movwf	FSR
	bankisel	BUFFER_SER
	movf	AUXC,W
	subwf	INDF,W
	btfss	STATUS, Z
	goto	SysForLoopEnd13
;next
	movlw	3
	subwf	AUXB,W
	btfss	STATUS, C
	goto	SysForLoop13
SysForLoopEnd13
;if auxb = 3 then  ' estava 4
	movlw	3
	subwf	AUXB,W
	btfss	STATUS, Z
	goto	ENDIF66
;procura_tx_eep = auxa 'encontrou tx: retorna o endereo inicial do tx
	movf	AUXA,W
	movwf	PROCURA_TX_EEP
;exit sub
	return
;end if
ENDIF66
;auxa = auxa + 6'passo de incremento do primeiro lao = 7
	movlw	6
	addwf	AUXA,F
;next
	movlw	119
	subwf	AUXA,W
	btfss	STATUS, C
	goto	SysForLoop12
SysForLoopEnd12
;procura_tx_eep = 0xAA 'retorna que no encontrou
	movlw	170
	movwf	PROCURA_TX_EEP
;no encontrado
	return

;********************************************************************************

RECEBER_DADOS
;if flag_ok = 0 then
	btfsc	SYSBITVAR0,4
	goto	ENDIF14
;if flag0 = 0 then
	btfsc	SYSBITVAR0,3
	goto	ELSE15_1
;incrementa_hc_lc'
	call	INCREMENTA_HC_LC
;flag0 = 1
;else
	goto	ENDIF15
ELSE15_1
;if rf_in = 0 then
	btfsc	PORTA,4
	goto	ELSE16_1
;if lc > TMAX then'define como intervalo entre transmisses
	movf	LC,W
	sublw	20
	btfsc	STATUS, C
	goto	ELSE17_1
;qb = 0
	clrf	QB
;lc = 0
	clrf	LC
;hc = 0
	clrf	HC
;flag0 = 0 'reinicia contadores
	bcf	SYSBITVAR0,3
;else
	goto	ENDIF17
ELSE17_1
;rotate_right(buffer_Ser,8)'
;C = 0
	bcf	STATUS,C
;rotate  buffer_Ser(7) right 'rotaciona 1 bit a direita
	banksel	SYSBUFFER_SER_7
	rrf	SYSBUFFER_SER_7,F
;rotate  buffer_Ser(6) right 'rotaciona 1 bit a direita
	rrf	SYSBUFFER_SER_6,F
;rotate  buffer_Ser(5) right 'rotaciona 1 bit a direita
	rrf	SYSBUFFER_SER_5,F
;rotate  buffer_Ser(4) right 'rotaciona 1 bit a direita
	rrf	SYSBUFFER_SER_4,F
;rotate  buffer_Ser(3) right 'rotaciona 1 bit a direita
	rrf	SYSBUFFER_SER_3,F
;rotate  buffer_Ser(2) right 'rotaciona 1 bit a direita
	rrf	SYSBUFFER_SER_2,F
;rotate  buffer_Ser(1) right 'rotaciona 1 bit a direita
	rrf	SYSBUFFER_SER_1,F
;rotate  buffer_Ser(0) right 'rotaciona 1 bit a direita
	rrf	SYSBUFFER_SER_0,F
;if lc > hc then
	banksel	LC
	movf	LC,W
	subwf	HC,W
	btfsc	STATUS, C
	goto	ENDIF18
;buffer_Ser(7) = buffer_Ser(7) or 0x80'
	movlw	128
	banksel	SYSBUFFER_SER_7
	iorwf	SYSBUFFER_SER_7,F
;else 'opcional
;buffer_Ser(7) = buffer_Ser(7) and 0x7f'opcional
;end if
ENDIF18
;flag0 = 0''prepara para pegar novo bit
	banksel	SYSBITVAR0
	bcf	SYSBITVAR0,3
;hc = 0'
	clrf	HC
;lc = 0'
	clrf	LC
;qb = qb + 1'
	incf	QB,F
;if qb >= NUMERO_BITS_A_RECEBER then
	movlw	64
	subwf	QB,W
	btfss	STATUS, C
	goto	ENDIF19
;qb = 0'
	clrf	QB
;if flag_r = 0 then
	btfsc	SYSBITVAR0,5
	goto	ELSE20_1
;flag_r = 1'
	bsf	SYSBITVAR0,5
;for auxb = 0 to 7
	movlw	255
	movwf	AUXB
SysForLoop6
	incf	AUXB,F
;buffer_Key(auxb) = buffer_Ser(auxb)''salva recepo para comparao
	movlw	low(BUFFER_SER)
	addwf	AUXB,W
	movwf	FSR
	bankisel	BUFFER_SER
	movf	INDF,W
	movwf	SysArrayTemp2
	movwf	SysArrayTemp1
	movlw	low(BUFFER_KEY)
	addwf	AUXB,W
	movwf	FSR
	bankisel	BUFFER_KEY
	movf	SysArrayTemp1,W
	movwf	INDF
;next
	movlw	7
	subwf	AUXB,W
	btfss	STATUS, C
	goto	SysForLoop6
SysForLoopEnd6
;else
	goto	ENDIF20
ELSE20_1
;for auxb = 0 to 7
	movlw	255
	movwf	AUXB
SysForLoop7
	incf	AUXB,F
;if buffer_Key(auxb) <> buffer_Ser(auxb) then exit for'
	movlw	low(BUFFER_KEY)
	addwf	AUXB,W
	movwf	FSR
	bankisel	BUFFER_KEY
	movf	INDF,W
	movwf	SysArrayTemp1
	movlw	low(BUFFER_SER)
	addwf	AUXB,W
	movwf	FSR
	bankisel	BUFFER_SER
	movf	INDF,W
	movwf	SysArrayTemp2
	subwf	SYSARRAYTEMP1,W
	btfss	STATUS, Z
	goto	SysForLoopEnd7
;next
	movlw	7
	subwf	AUXB,W
	btfss	STATUS, C
	goto	SysForLoop7
SysForLoopEnd7
;if auxb = 7 then
	movlw	7
	subwf	AUXB,W
	btfsc	STATUS, Z
;flag_ok = 1''recebeu uma transmisso completa
	bsf	SYSBITVAR0,4
;end if
;flag_r = 0'
	bcf	SYSBITVAR0,5
;end if
ENDIF20
;end if
ENDIF19
;end if'else
ENDIF17
;else
	goto	ENDIF16
ELSE16_1
;incrementa_hc_lc'
	call	INCREMENTA_HC_LC
;end if
ENDIF16
;end if' else{'flag0==1
ENDIF15
;end if
ENDIF14
	return

;********************************************************************************

SYSCOMPEQUAL
;Dim SysByteTempA, SysByteTempB, SysByteTempX as byte
;clrf SysByteTempX
	clrf	SYSBYTETEMPX
;movf SysByteTempA, W
	movf	SYSBYTETEMPA, W
;subwf SysByteTempB, W
	subwf	SYSBYTETEMPB, W
;btfsc STATUS, Z
	btfsc	STATUS, Z
;comf SysByteTempX,F
	comf	SYSBYTETEMPX,F
	return

;********************************************************************************

SYSCOMPEQUAL32
;dim SysLongTempA as long
;dim SysLongTempB as long
;dim SysByteTempX as byte
;clrf SysByteTempX
	clrf	SYSBYTETEMPX
;Test low, exit if false
;movf SysLongTempA, W
	movf	SYSLONGTEMPA, W
;subwf SysLongTempB, W
	subwf	SYSLONGTEMPB, W
;btfss STATUS, Z
	btfss	STATUS, Z
;return
	return
;Test high, exit if false
;movf SysLongTempA_H, W
	movf	SYSLONGTEMPA_H, W
;subwf SysLongTempB_H, W
	subwf	SYSLONGTEMPB_H, W
;btfss STATUS, Z
	btfss	STATUS, Z
;return
	return
;Test upper, exit if false
;movf SysLongTempA_U, W
	movf	SYSLONGTEMPA_U, W
;subwf SysLongTempB_U, W
	subwf	SYSLONGTEMPB_U, W
;btfss STATUS, Z
	btfss	STATUS, Z
;return
	return
;Test exp, exit if false
;movf SysLongTempA_E, W
	movf	SYSLONGTEMPA_E, W
;subwf SysLongTempB_E, W
	subwf	SYSLONGTEMPB_E, W
;btfss STATUS, Z
	btfss	STATUS, Z
;return
	return
;comf SysByteTempX,F
	comf	SYSBYTETEMPX,F
	return

;********************************************************************************

SYSCOMPLESSTHAN16
;dim SysWordTempA as word
;dim SysWordTempB as word
;dim SysByteTempX as byte
;clrf SysByteTempX
	clrf	SYSBYTETEMPX
;Test High, exit if more
;movf SysWordTempA_H,W
	movf	SYSWORDTEMPA_H,W
;subwf SysWordTempB_H,W
	subwf	SYSWORDTEMPB_H,W
;btfss STATUS,C
	btfss	STATUS,C
;return
	return
;Test high, exit true if less
;movf SysWordTempB_H,W
	movf	SYSWORDTEMPB_H,W
;subwf SysWordTempA_H,W
	subwf	SYSWORDTEMPA_H,W
;btfss STATUS,C
	btfss	STATUS,C
;goto SCLT16True
	goto	SCLT16TRUE
;Test Low, exit if more or equal
;movf SysWordTempB,W
	movf	SYSWORDTEMPB,W
;subwf SysWordTempA,W
	subwf	SYSWORDTEMPA,W
;btfsc STATUS,C
	btfsc	STATUS,C
;return
	return
SCLT16TRUE
;comf SysByteTempX,F
	comf	SYSBYTETEMPX,F
	return

;********************************************************************************

SYSCOMPLESSTHAN32
;dim SysLongTempA as long
;dim SysLongTempB as long
;dim SysByteTempX as byte
;clrf SysByteTempX
	clrf	SYSBYTETEMPX
;Test Exp, exit if more
;movf SysLongTempA_E,W
	movf	SYSLONGTEMPA_E,W
;subwf SysLongTempB_E,W
	subwf	SYSLONGTEMPB_E,W
;btfss STATUS,C
	btfss	STATUS,C
;return
	return
;If not more and not zero, is less
;btfss STATUS,Z
	btfss	STATUS,Z
;goto SCLT32True
	goto	SCLT32TRUE
;Test Upper, exit if more
;movf SysLongTempA_U,W
	movf	SYSLONGTEMPA_U,W
;subwf SysLongTempB_U,W
	subwf	SYSLONGTEMPB_U,W
;btfss STATUS,C
	btfss	STATUS,C
;return
	return
;If not more and not zero, is less
;btfss STATUS,Z
	btfss	STATUS,Z
;goto SCLT32True
	goto	SCLT32TRUE
;Test High, exit if more
;movf SysLongTempA_H,W
	movf	SYSLONGTEMPA_H,W
;subwf SysLongTempB_H,W
	subwf	SYSLONGTEMPB_H,W
;btfss STATUS,C
	btfss	STATUS,C
;return
	return
;If not more and not zero, is less
;btfss STATUS,Z
	btfss	STATUS,Z
;goto SCLT32True
	goto	SCLT32TRUE
;Test Low, exit if more or equal
;movf SysLongTempB,W
	movf	SYSLONGTEMPB,W
;subwf SysLongTempA,W
	subwf	SYSLONGTEMPA,W
;btfsc STATUS,C
	btfsc	STATUS,C
;return
	return
SCLT32TRUE
;comf SysByteTempX,F
	comf	SYSBYTETEMPX,F
	return

;********************************************************************************

SYSEPREAD
;Variable alias
;Dim EEAddress Alias EEADR
;Dim EEDataValue Alias EEDATL_REF
;Disable interrupt
;IntOff
;Select data memory
;Read
;SET RD ON
	banksel	EECON1
	bsf	EECON1,RD
;Restore interrupt
;IntOn
	banksel	STATUS
	return

;********************************************************************************

TABELA_Z
	movlw	17
	subwf	SysStringA, W
	btfsc	STATUS, C
	retlw	0
	movf	SysStringA, W
	addlw	low TableTABELA_Z
	movwf	SysStringA
	movlw	high TableTABELA_Z
	btfsc	STATUS, C
	addlw	1
	movwf	PCLATH
	movf	SysStringA, W
	movwf	PCL
TableTABELA_Z
	retlw	16
	retlw	4
	retlw	7
	retlw	5
	retlw	9
	retlw	10
	retlw	1
	retlw	11
	retlw	12
	retlw	0
	retlw	13
	retlw	14
	retlw	2
	retlw	15
	retlw	8
	retlw	6
	retlw	3

;********************************************************************************

TEMPORIZA_PROG_MODE
;if led = 1 then
	btfss	PORTB,1
	goto	ENDIF12
;cntprog = cntprog + 1'
	incf	CNTPROG,F
	btfsc	STATUS,Z
	incf	CNTPROG_H,F
	btfsc	STATUS,Z
	incf	CNTPROG_U,F
	btfsc	STATUS,Z
	incf	CNTPROG_E,F
;if cntprog = TEMPO_MAX_PROG_MODE then
	movf	CNTPROG,W
	movwf	SysLONGTempA
	movf	CNTPROG_H,W
	movwf	SysLONGTempA_H
	movf	CNTPROG_U,W
	movwf	SysLONGTempA_U
	movf	CNTPROG_E,W
	movwf	SysLONGTempA_E
	movlw	96
	movwf	SysLONGTempB
	movlw	91
	movwf	SysLONGTempB_H
	movlw	3
	movwf	SysLONGTempB_U
	clrf	SysLONGTempB_E
	call	SysCompEqual32
	btfss	SysByteTempX,0
	goto	ENDIF13
;led = 0'
	bcf	PORTB,1
;cntprog = 0'
	clrf	CNTPROG
	clrf	CNTPROG_H
	clrf	CNTPROG_U
	clrf	CNTPROG_E
;end if
ENDIF13
;end if
ENDIF12
	return

;********************************************************************************

TEMPORIZA_RELE
;Frmula: VALOR_ADVERTENCIA = tempo em segundos/(65538 *8 * 0,000001)
;tempo para 60 segundos:
;VALOR_ADVERTENCIA = 60/(65538 * 8 * 0,000001)
;VALOR_ADVERTENCIA = 114,44  (use 114)
;if rele = 0 then exit sub
	btfss	PORTA,2
	return
;if TMR1IF = 0 then exit sub
	btfss	PIR1,TMR1IF
	return
;TMR1IF = 0'
	bcf	PIR1,TMR1IF
;temp_desl = temp_desl + 1'
	incf	TEMP_DESL,F
	btfsc	STATUS,Z
	incf	TEMP_DESL_H,F
;if temp_desl > VALOR_ADVERTENCIA  then fled_ad = 1' 'seta flag pisca led de advertncia
	movf	TEMP_DESL,W
	movwf	SysWORDTempB
	movf	TEMP_DESL_H,W
	movwf	SysWORDTempB_H
	movlw	171
	movwf	SysWORDTempA
	clrf	SysWORDTempA_H
	call	SysCompLessThan16
	btfsc	SysByteTempX,0
	bsf	SYSBITVAR0,1
;if fled_ad = 1 then led_ad = not led_ad 'inverte estado do led (piscando)
	btfss	SYSBITVAR0,1
	goto	ENDIF48
	clrf	SysTemp1
	btfsc	PORTB,3
	incf	SysTemp1,F
	comf	SysTemp1,F
	bcf	PORTB,3
	btfsc	SysTemp1,0
	bsf	PORTB,3
ENDIF48
;if temp_desl > VALOR_BLOQUEIO then
	movf	TEMP_DESL,W
	movwf	SysWORDTempB
	movf	TEMP_DESL_H,W
	movwf	SysWORDTempB_H
	movlw	86
	movwf	SysWORDTempA
	movlw	1
	movwf	SysWORDTempA_H
	call	SysCompLessThan16
	btfss	SysByteTempX,0
	goto	ENDIF49
;rele = 0''desliga rele
	bcf	PORTA,2
;temp_s = 0;
	clrf	TEMP_S
;temp_sirene_on = 0'zera contador de tempo de desligamento
	clrf	TEMP_SIRENE_ON
;fled_sir = 1'seta flag de tempo de sirene disparada
	bsf	SYSBITVAR0,0
;led_ad = 0'desliga led de advertncia
	bcf	PORTB,3
;fled_ad = 0;//reseta flag do led de advertncia
	bcf	SYSBITVAR0,1
;led_bloq = 1''liga led de advertncia
	bsf	PORTB,2
;temp_desl = 0'
	clrf	TEMP_DESL
	clrf	TEMP_DESL_H
;end if
ENDIF49
	return

;********************************************************************************

TEMPORIZA_SIRENE
;se rele est ligado sai da rotina
;if rele = 1 then exit sub
	btfsc	PORTA,2
	return
;sirene j disparou 1 vez
;if sirene = 0 then exit sub
	btfss	PORTA,1
	return
;if TMR1IF = 0 then  exit sub
	btfss	PIR1,TMR1IF
	return
;TMR1IF = 0;
	bcf	PIR1,TMR1IF
;temp_s = temp_s + 1
	incf	TEMP_S,F
;if temp_s >= VALOR_UM_MINUTO then
	movlw	114
	subwf	TEMP_S,W
	btfss	STATUS, C
	goto	ENDIF55
;temp_s = 0
	clrf	TEMP_S
;temp_sirene_on = temp_sirene_on + 1
	incf	TEMP_SIRENE_ON,F
;if temp_sirene_on >= VALOR_DESLIGA_SIRENE then
	movlw	10
	subwf	TEMP_SIRENE_ON,W
	btfss	STATUS, C
	goto	ENDIF56
;sirene = 0'desliga a sirene
	bcf	PORTA,1
;escreve_eep(127,0)'grava '0'
	movlw	127
	movwf	END1
	clrf	VALOR1
	call	ESCREVE_EEP
;temp_s = 0
	clrf	TEMP_S
;temp_sirene_on = 0'zera contador de tempo de desligamento
	clrf	TEMP_SIRENE_ON
;end if
ENDIF56
;end if
ENDIF55
	return

;********************************************************************************

TESTA_BOTAO
;if btn = 0 then
	btfsc	PORTB,0
	goto	ENDIF3
;delay50ms'
	call	DELAY50MS
;cntap = 0'
	clrf	CNTAP
;do while btn = 0
SysDoLoop_S2
	btfsc	PORTB,0
	goto	SysDoLoop_E2
;led = 1'
	bsf	PORTB,1
;cntap = cntap + 1'
	incf	CNTAP,F
;delay50ms'
	call	DELAY50MS
;if cntap > TEMPO_APAGAMENTO then
	movf	CNTAP,W
	sublw	100
	btfsc	STATUS, C
	goto	ENDIF4
;for auxa = 0 to 127'
	movlw	255
	movwf	AUXA
SysForLoop1
	incf	AUXA,F
;escreve_eep(auxa , 0xff)' 'seta todas as outras posies da EEprom
	movf	AUXA,W
	movwf	END1
	movlw	255
	movwf	VALOR1
	call	ESCREVE_EEP
	movf	END1,W
	movwf	AUXA
;next
	movlw	127
	subwf	AUXA,W
	btfss	STATUS, C
	goto	SysForLoop1
SysForLoopEnd1
;cntap = 0'
	clrf	CNTAP
;for auxa = 1 to 6 'pisca led 5 vezes para indicar apagamento
	clrf	AUXA
SysForLoop2
	incf	AUXA,F
;led = 1'
	bsf	PORTB,1
;delayp'
	call	DELAYP
;led = 0'
	bcf	PORTB,1
;delayp'
	call	DELAYP
;next
	movlw	6
	subwf	AUXA,W
	btfss	STATUS, C
	goto	SysForLoop2
SysForLoopEnd2
;do while btn = 0
SysDoLoop_S3
	btfsc	PORTB,0
	goto	SysDoLoop_E3
;wait 100 ms 'aguarda soltar boto
	movlw	100
	movwf	SysWaitTempMS
	clrf	SysWaitTempMS_H
	call	Delay_MS
;loop
	goto	SysDoLoop_S3
SysDoLoop_E3
;end if
ENDIF4
;loop
	goto	SysDoLoop_S2
SysDoLoop_E2
;end if
ENDIF3
	return

;********************************************************************************


 END
